/*
 *
 * AFRO-PRODUCTIONS.COM
 *	Reverse Pimpage 2.2 (c)
 *
 * Author: Matt Miller
 *              (warped@hick.org)
 *
 * Terminal emulation done by Tommy (dantar@dantar.com)
 *
 *  Reverse Pimpage version 2 is a total rewrite of the original version.  It has been optimized and many new
 * (much needed) things have been added.  The new code is much more efficient.
 *
 * main.c: this file is responsible for deciding what we do, whether we're client/server, and what arguments to use.
 *
 * Patched by: Andrea Biardi (LOGICOMM)
 *		(andrea.biardi@logicomm.it)
 * 2003-09-03
 *	- Fix options -E, -f; rework help text; explicitely add -? to getopt()
 *	- Replace stdout with stderr where appropriate
 *	- Add install and uninstall targets to Makefile.in, enable use of --prefix and/or --bindir
 *	- Make DEFAULT_TIMEOUT a constant
 */ 


#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>

#include <arpa/telnet.h>

#include "struct.h"
#include "sock.h"

struct global_template global;

void fnCloseSafe();
void fnSendBG();
void fnDoConnect();
void fnPrintVersion();
void fnPrintUsage(char **);
void fnInitGlobal();


int main(int argc, char *argv[])
{
	extern char *optarg;
	int c;

	fnInitGlobal();

	while ((c = getopt(argc,argv,"?cE:e:fg:h:P:p:st:v")) != EOF)
	{
		switch (c)
		{
			case 'c':
				global.client = 1;
				global.server = 0;
				break;
			case 'E':
				global.escapechar = optarg[0];
				break;
			case 'e':
				global.expectstr = optarg;
				break;
			case 'f':
				global.fork = 1;
				break;
			case 'g':
				global.gateway = optarg;
				break;
			case 'h':
				global.remotehost = optarg;
				break;
			case 'P':
				global.proto = optarg;
				break;
			case 'p':
				global.remoteport = atoi(optarg);
				break;
			case 's':
				global.server = 1;
				global.client = 0;
				break;
			case 't':
				global.timeout = atoi(optarg);
				break;
			case 'v':
				fnPrintVersion();
				return 0;
			default:
				fnPrintUsage(argv);
		}
	}

	signal(SIGINT,fnCloseSafe);
	signal(SIGTSTP,fnSendBG);

	if (global.client)
	{
		if (!global.remotehost)
		{
			fprintf(stderr,"Client suggested, but host not given.\n");
			fnPrintUsage(argv);
			return -1;
		}
		else
		{
			if (global.fork) {
				int pid = fork();
				if (pid == -1)
					fprintf(stderr,"ERROR: unable to fork, continuing...\n");
				else if (pid != 0)
				{
					fprintf(stderr,"Successfully forked into pid %i\n",pid);
					exit(-1);
				}
			} else {
				fprintf(stderr,"Reverse pimpage ready.\n");
			}
			while (1)
				fnDoConnect();
		}	
	}
	else if (global.server)
	{
		char tmpchar,prevchar;

		global.serversock = fnSockServerInit(global.remoteport);

		if (global.serversock == -1)
			return -1;

		fprintf(stderr,"Connection established.\n");

		if (global.escapechar <= 31)
		{
			prevchar = '^';
			tmpchar = global.escapechar + 64;
		}
		else
		{
			prevchar = ' ';
			tmpchar = global.escapechar;
		}

		fprintf(stderr,"Escape character is '%c%c'\n",prevchar,tmpchar);

		global.connected = 1;

		fnMonitorServerSock();
	}	
	else
		fprintf(stderr,"FATAL: Neither server or client defined, exiting (try -? for help).\n");
		
	return 1;
}

void fnPrintVersion()
{
	fprintf(stdout,"Reverse Pimpage v2.2 by: Matt Miller (warped@hick.org)\n");
	fprintf(stdout,"\tBug fixes and Term Emul. by Tommy (dantar@dantar.com)\n");
	fprintf(stdout,"\tPatches by A. Biardi (andrea.biardi@logicomm.it)\n");
	return;
}

void fnPrintUsage(char **argv)
{
	fprintf(stdout,"Usage: %s [-?|-c|-s|-v] [-f] [-E escapechar] [-e expectstr] [-g gateway] [-h host] [-P protocol] [-p port] [-t timeout]\n",argv[0]);
	fprintf(stdout,"\t-?\tShow this stuff and exit\n");
	fprintf(stdout,"\t-c\tClient mode, host needed (see -h below)\n");
	fprintf(stdout,"\t-s\tServer mode (-h not needed)\n");
	fprintf(stdout,"\t-v\tShow version information and exit\n");
	fprintf(stdout,"\t-f\tFork, when used as a client\n");
	fprintf(stdout,"\t-E\tEscape character\n");
	fprintf(stdout,"\t-e\tExpect string to expect from gateway (will send remotehost)\n");
	fprintf(stdout,"\t-g\tOptional gateway\n");
	fprintf(stdout,"\t-h\tRemote host, when used as a client\n");
	fprintf(stdout,"\t-P\tProtocol to use, either telnet or none\n");
	fprintf(stdout,"\t-p\tPort to use (default: %d)\n",DEFAULT_PORT);
	fprintf(stdout,"\t-t\tTimeout for client to reconnect, in seconds (default: %d)\n",DEFAULT_TIMEOUT);
	exit(-1);
}

void fnInitGlobal()
{
	global.remotehost = NULL;
	global.expectstr = NULL;
	global.proto = "std";
	global.gateway = NULL;
	global.timeout = DEFAULT_TIMEOUT;
	global.client = 0;
	global.connected = 0;
	global.fork = 0;
	global.server = 0;
	global.found = 0;
	global.escapechar = 30;
	global.telnetsock = -1;
	global.serversock = -1;
	global.clientsock = -1;
	global.remoteport = DEFAULT_PORT;
}

void fnCloseSafe()
{
	char tmp[2];

	if (global.connected == 1)
	{
		tmp[0] = IAC;
		tmp[1] = IP;
		write(global.clientsock,tmp,2);
	}
	else
	{
		fprintf(stderr,"Exiting...\n");
		exit(-1);
	}
}

void fnSendBG()
{
	char tmp[2];

	if (global.connected == 1)
	{
		tmp[0] = IAC;
		tmp[1] = SUSP;
		write(global.clientsock,tmp,2);
	}
}	

void fnDoConnect()
{
	global.clientsock = fnSockClientInit((!global.gateway)?global.remotehost:global.gateway,global.remoteport);

	if (global.clientsock != -1)
	{
		if (!global.gateway)
			global.telnetsock = fnSockClientInit("127.0.0.1",DEFAULT_TELNET_PORT);

		global.connected = 1;

		fnMonitorClientSock();
	}

	global.found = 0;
	global.connected = 0;
	global.clientsock = -1;
	global.telnetsock = -1;

	sleep(global.timeout);
}
